@@ -10,10 +10,9 @@ class TestDocument < JekyllUnitTest
   def setup_encoded_document(filename)
     site = fixture_site("collections" => ["encodings"])
     site.process
-    Document.new(site.in_source_dir(File.join("_encodings", filename)), {
-      :site       => site,
-      :collection => site.collections["encodings"],
-    }).tap(&:read)
+    Document.new(site.in_source_dir(File.join("_encodings", filename)),
+                 :site       => site,
+                 :collection => site.collections["encodings"]).tap(&:read)
   end
 
   def setup_document_with_dates(filename)
@@ -21,19 +20,18 @@ class TestDocument < JekyllUnitTest
     site.process
     docs = nil
     with_env("TZ", "UTC") do
-      docs = Document.new(site.in_source_dir(File.join("_dates", filename)), {
-        :site       => site,
-        :collection => site.collections["dates"],
-      }).tap(&:read)
+      docs = Document.new(site.in_source_dir(File.join("_dates", filename)),
+                          :site       => site,
+                          :collection => site.collections["dates"]).tap(&:read)
     end
     docs
   end
 
   context "a document in a collection" do
     setup do
-      @site = fixture_site({
-        "collections" => ["methods"],
-      })
+      @site = fixture_site(
+        "collections" => ["methods"]
+      )
       @site.process
       @document = @site.collections["methods"].docs.detect do |d|
         d.relative_path == "_methods/configuration.md"
@@ -41,7 +39,7 @@ class TestDocument < JekyllUnitTest
     end
 
     should "exist" do
-      assert !@document.nil?
+      refute_nil @document
     end
 
     should "know its relative path" do
@@ -60,8 +58,12 @@ class TestDocument < JekyllUnitTest
       assert_equal "configuration", @document.basename_without_ext
     end
 
+    should "know its type" do
+      assert_equal :methods, @document.type
+    end
+
     should "know whether it's a YAML file" do
-      assert_equal false, @document.yaml_file?
+      refute @document.yaml_file?
     end
 
     should "know its data" do
@@ -69,6 +71,12 @@ class TestDocument < JekyllUnitTest
       assert_equal "foo.bar", @document.data["whatever"]
     end
 
+    should "know its date" do
+      assert_nil @document.data["date"]
+      assert_equal Time.now.strftime("%Y/%m/%d"), @document.date.strftime("%Y/%m/%d")
+      assert_equal Time.now.strftime("%Y/%m/%d"), @document.to_liquid["date"].strftime("%Y/%m/%d")
+    end
+
     should "be jsonify-able" do
       page_json = @document.to_liquid.to_json
       parsed = JSON.parse(page_json)
@@ -110,9 +118,28 @@ class TestDocument < JekyllUnitTest
       assert_nil next_next_doc["previous"]["output"]
     end
 
+    context "with the basename (without extension) ending with dot(s)" do
+      setup do
+        @site = fixture_site("collections" => ["methods"])
+        @site.process
+        @document = @site.collections["methods"].docs.detect do |d|
+          d.relative_path == "_methods/trailing-dots...md"
+        end
+      end
+
+      should "render into the proper url" do
+        assert_equal "/methods/trailing-dots.html", @document.url
+
+        trailing_dots_doc = @site.posts.docs.detect do |d|
+          d.relative_path == "_posts/2018-10-12-trailing-dots...markdown"
+        end
+        assert_equal "/2018/10/12/trailing-dots.html", trailing_dots_doc.url
+      end
+    end
+
     context "with YAML ending in three dots" do
       setup do
-        @site = fixture_site({ "collections" => ["methods"] })
+        @site = fixture_site("collections" => ["methods"])
         @site.process
         @document = @site.collections["methods"].docs.detect do |d|
           d.relative_path == "_methods/yaml_with_dots.md"
@@ -126,17 +153,34 @@ class TestDocument < JekyllUnitTest
     end
 
     should "output the collection name in the #to_liquid method" do
-      assert_equal @document.to_liquid["collection"], "methods"
+      assert_equal "methods", @document.to_liquid["collection"]
     end
 
     should "output its relative path as path in Liquid" do
-      assert_equal @document.to_liquid["path"], "_methods/configuration.md"
+      assert_equal "_methods/configuration.md", @document.to_liquid["path"]
+    end
+
+    context "when rendered with Liquid" do
+      should "respect the front matter definition" do
+        site = fixture_site("collections" => ["roles"]).tap(&:process)
+        docs = site.collections["roles"].docs
+
+        # Ruby context: doc.basename is aliased as doc.to_liquid["name"] by default.
+
+        document = docs.detect { |d| d.relative_path == "_roles/unnamed.md" }
+        assert_equal "unnamed.md", document.basename
+        assert_equal "unnamed.md", document.to_liquid["name"]
+
+        document = docs.detect { |d| d.relative_path == "_roles/named.md" }
+        assert_equal "named.md", document.basename
+        assert_equal "launcher", document.to_liquid["name"]
+      end
     end
   end
 
   context "a document as part of a collection with front matter defaults" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => ["slides"],
         "defaults"    => [{
           "scope"  => { "path" => "", "type" => "slides" },
@@ -145,10 +189,10 @@ class TestDocument < JekyllUnitTest
               "key" => "myval",
             },
           },
-        },],
-      })
+        }]
+      )
       @site.process
-      @document = @site.collections["slides"].docs.select { |d| d.is_a?(Document) }.first
+      @document = @site.collections["slides"].docs.find { |d| d.is_a?(Document) }
     end
 
     should "know the front matter defaults" do
@@ -166,7 +210,7 @@ class TestDocument < JekyllUnitTest
 
   context "a document as part of a collection with overridden default values" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => ["slides"],
         "defaults"    => [{
           "scope"  => { "path" => "", "type" => "slides" },
@@ -176,8 +220,8 @@ class TestDocument < JekyllUnitTest
               "test2" => "default1",
             },
           },
-        },],
-      })
+        }]
+      )
       @site.process
       @document = @site.collections["slides"].docs[1]
     end
@@ -194,7 +238,7 @@ class TestDocument < JekyllUnitTest
 
   context "a document as part of a collection with valid path" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => ["slides"],
         "defaults"    => [{
           "scope"  => { "path" => "_slides", "type" => "slides" },
@@ -203,8 +247,8 @@ class TestDocument < JekyllUnitTest
               "key" => "value123",
             },
           },
-        },],
-      })
+        }]
+      )
       @site.process
       @document = @site.collections["slides"].docs.first
     end
@@ -218,7 +262,7 @@ class TestDocument < JekyllUnitTest
 
   context "a document as part of a collection with invalid path" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => ["slides"],
         "defaults"    => [{
           "scope"  => { "path" => "somepath", "type" => "slides" },
@@ -227,8 +271,8 @@ class TestDocument < JekyllUnitTest
               "key" => "myval",
             },
           },
-        },],
-      })
+        }]
+      )
       @site.process
       @document = @site.collections["slides"].docs.first
     end
@@ -242,9 +286,9 @@ class TestDocument < JekyllUnitTest
 
   context "a document in a collection with a custom permalink" do
     setup do
-      @site = fixture_site({
-        "collections" => ["slides"],
-      })
+      @site = fixture_site(
+        "collections" => ["slides"]
+      )
       @site.process
       @document = @site.collections["slides"].docs[2]
       @dest_file = dest_dir("slide/3/index.html")
@@ -261,15 +305,15 @@ class TestDocument < JekyllUnitTest
 
   context "a document in a collection with custom filename permalinks" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "slides" => {
             "output"    => true,
             "permalink" => "/slides/test/:name",
           },
         },
-        "permalink"   => "pretty",
-      })
+        "permalink"   => "pretty"
+      )
       @site.process
       @document = @site.collections["slides"].docs[0]
       @dest_file = dest_dir("slides/test/example-slide-1.html")
@@ -290,13 +334,13 @@ class TestDocument < JekyllUnitTest
 
   context "a document in a collection with pretty permalink style" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "slides" => {
             "output" => true,
           },
-        },
-      })
+        }
+      )
       @site.permalink_style = :pretty
       @site.process
       @document = @site.collections["slides"].docs[0]
@@ -314,13 +358,13 @@ class TestDocument < JekyllUnitTest
 
   context "a document in a collection with cased file name" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "slides" => {
             "output" => true,
           },
-        },
-      })
+        }
+      )
       @site.permalink_style = :pretty
       @site.process
       @document = @site.collections["slides"].docs[7]
@@ -334,13 +378,13 @@ class TestDocument < JekyllUnitTest
 
   context "a document in a collection with cased file name" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "slides" => {
             "output" => true,
           },
-        },
-      })
+        }
+      )
       @site.process
       @document = @site.collections["slides"].docs[6]
       @dest_file = dest_dir("slides/example-slide-7.php")
@@ -365,14 +409,14 @@ class TestDocument < JekyllUnitTest
 
   context "documents in a collection with custom title permalinks" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "slides" => {
             "output"    => true,
             "permalink" => "/slides/:title",
           },
-        },
-      })
+        }
+      )
       @site.process
       @document = @site.collections["slides"].docs[3]
       @document_without_slug = @site.collections["slides"].docs[4]
@@ -410,9 +454,9 @@ class TestDocument < JekyllUnitTest
 
   context "document with a permalink with dots & a trailing slash" do
     setup do
-      @site = fixture_site({ "collections" => {
+      @site = fixture_site("collections" => {
         "with.dots" => { "output" => true },
-      }, })
+      })
       @site.process
       @document = @site.collections["with.dots"].docs.last
       @dest_file = dest_dir("with.dots", "permalink.with.slash.tho", "index.html")
@@ -433,13 +477,13 @@ class TestDocument < JekyllUnitTest
 
   context "documents in a collection" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "slides" => {
             "output" => true,
           },
-        },
-      })
+        }
+      )
       @site.process
       @files = @site.collections["slides"].docs
     end
@@ -465,13 +509,13 @@ class TestDocument < JekyllUnitTest
 
   context "a static file in a collection" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "slides" => {
             "output" => true,
           },
-        },
-      })
+        }
+      )
       @site.process
       @document = @site.collections["slides"].files.find do |doc|
         doc.relative_path == "_slides/octojekyll.png"
@@ -480,7 +524,7 @@ class TestDocument < JekyllUnitTest
     end
 
     should "be a static file" do
-      assert_equal true, @document.is_a?(StaticFile)
+      assert @document.is_a?(StaticFile)
     end
 
     should "be set to write" do
@@ -488,7 +532,7 @@ class TestDocument < JekyllUnitTest
     end
 
     should "be in the list of docs_to_write" do
-      assert @site.docs_to_write.include?(@document)
+      assert_includes @site.docs_to_write, @document
     end
 
     should "be output in the correct place" do
@@ -498,13 +542,13 @@ class TestDocument < JekyllUnitTest
 
   context "a document in a collection with non-alphabetic file name" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "methods" => {
             "output" => true,
           },
-        },
-      })
+        }
+      )
       @site.process
       @document = @site.collections["methods"].docs.find do |doc|
         doc.relative_path == "_methods/escape-+ #%20[].md"
@@ -521,19 +565,19 @@ class TestDocument < JekyllUnitTest
     end
 
     should "be output in the correct place" do
-      assert_equal true, File.file?(@dest_file)
+      assert File.file?(@dest_file)
     end
   end
 
   context "a document in a collection with dash-separated numeric file name" do
     setup do
-      @site = fixture_site({
+      @site = fixture_site(
         "collections" => {
           "methods" => {
             "output" => true,
           },
-        },
-      })
+        }
+      )
       @site.process
       @document = @site.collections["methods"].docs.find do |doc|
         doc.relative_path == "_methods/3940394-21-9393050-fifif1323-test.md"
@@ -550,7 +594,7 @@ class TestDocument < JekyllUnitTest
     end
 
     should "be output in the correct place" do
-      assert_equal true, File.file?(@dest_file)
+      assert File.file?(@dest_file)
     end
   end
 
@@ -582,6 +626,10 @@ class TestDocument < JekyllUnitTest
     should "have the expected date" do
       assert_equal "2015/09/30", @document.data["date"].strftime("%Y/%m/%d")
     end
+
+    should "return the expected date via Liquid" do
+      assert_equal "2015/09/30", @document.to_liquid["date"].strftime("%Y/%m/%d")
+    end
   end
 
   context "a document with a date with time but without timezone" do
@@ -592,6 +640,10 @@ class TestDocument < JekyllUnitTest
     should "have the expected date" do
       assert_equal "2015/10/01", @document.data["date"].strftime("%Y/%m/%d")
     end
+
+    should "return the expected date via Liquid" do
+      assert_equal "2015/10/01", @document.to_liquid["date"].strftime("%Y/%m/%d")
+    end
   end
 
   context "a document with a date without time" do
@@ -602,5 +654,9 @@ class TestDocument < JekyllUnitTest
     should "have the expected date" do
       assert_equal "2015/10/01", @document.data["date"].strftime("%Y/%m/%d")
     end
+
+    should "return the expected date via Liquid" do
+      assert_equal "2015/10/01", @document.to_liquid["date"].strftime("%Y/%m/%d")
+    end
   end
 end
